@@ -52,6 +52,9 @@
 #include <QDesktopWidget>
 #include <QTcpSocket>
 #include <QMessageBox>
+#include <QTextDocumentFragment>
+#include <QClipboard>
+#include <QDesktopServices>
 
 /*!
  * \class SimulationOutputTree
@@ -245,6 +248,7 @@ SimulationOutputWidget::SimulationOutputWidget(SimulationOptions simulationOptio
   mpProgressBar->setAlignment(Qt::AlignHCenter);
   // Generated Files tab widget
   mpGeneratedFilesTabWidget = new QTabWidget;
+  mpGeneratedFilesTabWidget->setDocumentMode(true);
   mpGeneratedFilesTabWidget->setMovable(true);
   // Compilation Output TextBox
   mpCompilationOutputTextBox = new OutputPlainTextEdit;
@@ -278,8 +282,7 @@ SimulationOutputWidget::SimulationOutputWidget(SimulationOptions simulationOptio
   mGeneratedFilesList << "%1.makefile";
   // cpp-runtime generated files
   if (simulationOptions.getTargetLanguage().compare("Cpp") == 0) {
-    mGeneratedFilesList << "%1.bat"
-                        << "OMCpp%1.cpp"
+    mGeneratedFilesList << "OMCpp%1.cpp"
                         << "OMCpp%1.h"
                         << "OMCpp%1.exp"
                         << "OMCpp%1.lib"
@@ -412,12 +415,6 @@ SimulationOutputWidget::SimulationOutputWidget(SimulationOptions simulationOptio
   mpSimulationProcess = 0;
   setSimulationProcessKilled(false);
   mIsSimulationProcessRunning = false;
-
-  if (!mSimulationOptions.isReSimulate()) {
-    compileModel();
-  } else {
-    runSimulationExecutable();
-  }
 }
 
 /*!
@@ -452,6 +449,19 @@ SimulationOutputWidget::~SimulationOutputWidget()
   }
 }
 
+/*!
+ * \brief SimulationOutputWidget::start
+ * Starts the compilation/simulation.
+ */
+void SimulationOutputWidget::start()
+{
+  if (!mSimulationOptions.isReSimulate()) {
+    compileModel();
+  } else {
+    runSimulationExecutable();
+  }
+}
+
 void SimulationOutputWidget::addGeneratedFileTab(QString fileName)
 {
   QFile file(fileName);
@@ -629,9 +639,11 @@ void SimulationOutputWidget::runPostCompilation()
 void SimulationOutputWidget::postCompilationProcessStarted()
 {
   mIsPostCompilationProcessRunning = true;
-  mpProgressLabel->setText(tr("Post compiling %1.").arg(mSimulationOptions.getClassName()));
+  const QString progressStr = tr("Post compiling %1.").arg(mSimulationOptions.getClassName());
+  mpProgressLabel->setText(progressStr);
   mpProgressBar->setRange(0, 0);
   mpProgressBar->setTextVisible(false);
+  updateMessageTab(progressStr);
   mpCancelButton->setText(tr("Cancel Compilation"));
   mpCancelButton->setEnabled(true);
 }
@@ -697,12 +709,18 @@ void SimulationOutputWidget::postCompilationProcessFinished(int exitCode, QProce
   }
 }
 
-void SimulationOutputWidget::postCompilationProcessFinishedHelper(int /*exitCode*/, QProcess::ExitStatus /*exitStatus*/)
+void SimulationOutputWidget::postCompilationProcessFinishedHelper(int exitCode, QProcess::ExitStatus exitStatus)
 {
-  mpProgressLabel->setText(tr("Post compilation of %1 is finished.").arg(mSimulationOptions.getClassName()));
+  const QString progressStr = tr("Post compilation of %1 is finished.").arg(mSimulationOptions.getClassName());
+  mpProgressLabel->setText(progressStr);
   mpProgressBar->setRange(0, 1);
-  mpProgressBar->setValue(1);
   mpCancelButton->setEnabled(false);
+  if (exitStatus == QProcess::NormalExit && exitCode == 0) {
+    mpProgressBar->setValue(1);
+  } else {
+    mpProgressBar->setValue(0);
+  }
+  updateMessageTab(progressStr);
 }
 
 /*!
@@ -735,6 +753,25 @@ QString SimulationOutputWidget::getPathsFromBatFile(QString fileName) {
   return line;
 }
 
+/*!
+ * \brief SimulationOutputWidget::updateMessageTab
+ * Updates the corresponsing MessageTab.
+ */
+void SimulationOutputWidget::updateMessageTab(const QString &text)
+{
+  emit updateText(text);
+  emit updateProgressBar(mpProgressBar);
+}
+
+/*!
+ * \brief SimulationOutputWidget::updateMessageTabProgress
+ * Updates the progress bar of MessageTab
+ */
+void SimulationOutputWidget::updateMessageTabProgress()
+{
+  emit updateProgressBar(mpProgressBar);
+}
+
 /*!
  * \brief SimulationOutputWidget::runSimulationExecutable
  * Runs the simulation executable.
@@ -796,11 +833,12 @@ void SimulationOutputWidget::writeCompilationOutput(QString output, QColor color
 
 void SimulationOutputWidget::compilationProcessFinishedHelper(int exitCode, QProcess::ExitStatus exitStatus)
 {
-  mpProgressLabel->setText(tr("Compilation of %1 is finished.").arg(mSimulationOptions.getClassName()));
+  const QString progressStr = tr("Compilation of %1 is finished.").arg(mSimulationOptions.getClassName());
+  mpProgressLabel->setText(progressStr);
   mpProgressBar->setRange(0, 1);
-  mpProgressBar->setValue(1);
   mpCancelButton->setEnabled(false);
   if (exitStatus == QProcess::NormalExit && exitCode == 0) {
+    mpProgressBar->setValue(1);
     bool profiling = mSimulationOptions.getProfiling().compare(QStringLiteral("none")) != 0;
     if (mSimulationOptions.getBuildOnly() &&
         (OptionsDialog::instance()->getDebuggerPage()->getAlwaysShowTransformationsCheckBox()->isChecked() ||
@@ -808,7 +846,10 @@ void SimulationOutputWidget::compilationProcessFinishedHelper(int exitCode, QPro
       MainWindow::instance()->showTransformationsWidget(mSimulationOptions.getWorkingDirectory() + "/" + mSimulationOptions.getOutputFileName() + "_info.json", profiling);
     }
     MainWindow::instance()->getSimulationDialog()->showAlgorithmicDebugger(mSimulationOptions);
+  } else {
+    mpProgressBar->setValue(0);
   }
+  updateMessageTab(progressStr);
   mpArchivedSimulationItem->setStatus(Helper::finished);
   // remove the generated files
   if (mSimulationOptions.getBuildOnly()) {
@@ -853,11 +894,7 @@ void SimulationOutputWidget::deleteIntermediateCompilationFiles()
 void SimulationOutputWidget::writeSimulationOutput(QString output, StringHandler::SimulationMessageType type, bool textFormat)
 {
   if (textFormat) {
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
     QString escaped = QString(output).toHtmlEscaped();
-#else /* Qt4 */
-    QString escaped = Qt::escape(output);
-#endif
     output = QString("<message stream=\"stdout\" type=\"%1\" text=\"%2\" />")
         .arg(StringHandler::getSimulationMessageTypeString(type))
         .arg(escaped);
@@ -894,11 +931,13 @@ void SimulationOutputWidget::simulationProcessFinishedHelper()
   int exitCode = mpSimulationProcess->exitCode();
   QProcess::ExitStatus exitStatus = mpSimulationProcess->exitStatus();
   QString exitCodeStr = tr("Simulation process failed. Exited with code %1.").arg(Utilities::formatExitCode(exitCode));
+  QString progressStr;
   if (exitStatus == QProcess::NormalExit && exitCode == 0) {
     /* Ticket:4486
      * Don't print the success message since omc now outputs the success information.
      */
     //writeSimulationOutput(tr("Simulation process finished successfully."), StringHandler::OMEditInfo, true);
+    progressStr = tr("Simulation of %1 finished.").arg(mSimulationOptions.getClassName());
   } else {
     if (mpSimulationOutputHandler) {
       SimulationMessage *pSimulationMessage;
@@ -907,7 +946,7 @@ void SimulationOutputWidget::simulationProcessFinishedHelper()
       } else {
         pSimulationMessage = new SimulationMessage;
       }
-      pSimulationMessage->mStream = "stdout";
+      pSimulationMessage->mStream = "LOG_STDOUT";
       pSimulationMessage->mType = StringHandler::Error;
       pSimulationMessage->mLevel = 0;
 
@@ -919,10 +958,10 @@ void SimulationOutputWidget::simulationProcessFinishedHelper()
 
       mpSimulationOutputHandler->addSimulationMessage(pSimulationMessage);
     }
+    progressStr = tr("Simulation of %1 failed.").arg(mSimulationOptions.getClassName());
   }
-
-  mpProgressLabel->setText(tr("Simulation of %1 is finished.").arg(mSimulationOptions.getClassName()));
-  mpProgressBar->setValue(mpProgressBar->maximum());
+  mpProgressLabel->setText(progressStr);
+  updateMessageTab(progressStr);
   mpCancelButton->setEnabled(false);
   MainWindow::instance()->getSimulationDialog()->simulationProcessFinished(mSimulationOptions, mResultFileLastModifiedDateTime);
   mpArchivedSimulationItem->setStatus(Helper::finished);
@@ -944,30 +983,32 @@ void SimulationOutputWidget::simulationProcessFinishedHelper()
  */
 void SimulationOutputWidget::cancelCompilationOrSimulation()
 {
+  QString progressStr;
   if (isCompilationProcessRunning()) {
     setCompilationProcessKilled(true);
     mpCompilationProcess->kill();
-    mpProgressLabel->setText(tr("Compilation of %1 is cancelled.").arg(mSimulationOptions.getClassName()));
+    progressStr = tr("Compilation of %1 is cancelled.").arg(mSimulationOptions.getClassName());
     mpProgressBar->setRange(0, 1);
-    mpProgressBar->setValue(1);
+    mpProgressBar->setValue(0);
     mpCancelButton->setEnabled(false);
     mpArchivedSimulationItem->setStatus(Helper::finished);
   } else if (isPostCompilationProcessRunning()) {
     setPostCompilationProcessKilled(true);
     mpPostCompilationProcess->kill();
-    mpProgressLabel->setText(tr("Post compilation of %1 is cancelled.").arg(mSimulationOptions.getClassName()));
+    progressStr = tr("Post compilation of %1 is cancelled.").arg(mSimulationOptions.getClassName());
     mpProgressBar->setRange(0, 1);
-    mpProgressBar->setValue(1);
+    mpProgressBar->setValue(0);
     mpCancelButton->setEnabled(false);
     mpArchivedSimulationItem->setStatus(Helper::finished);
   } else if (isSimulationProcessRunning()) {
     setSimulationProcessKilled(true);
     mpSimulationProcess->kill();
-    mpProgressLabel->setText(tr("Simulation of %1 is cancelled.").arg(mSimulationOptions.getClassName()));
-    mpProgressBar->setValue(mpProgressBar->maximum());
+    progressStr = tr("Simulation of %1 is cancelled.").arg(mSimulationOptions.getClassName());
     mpCancelButton->setEnabled(false);
     mpArchivedSimulationItem->setStatus(Helper::finished);
   }
+  mpProgressLabel->setText(progressStr);
+  updateMessageTab(progressStr);
 }
 
 /*!
@@ -1058,9 +1099,11 @@ void SimulationOutputWidget::socketDisconnected()
 void SimulationOutputWidget::compilationProcessStarted()
 {
   mIsCompilationProcessRunning = true;
-  mpProgressLabel->setText(tr("Compiling %1. Please wait for a while.").arg(mSimulationOptions.getClassName()));
+  const QString progressStr = tr("Compiling %1. Please wait for a while.").arg(mSimulationOptions.getClassName());
+  mpProgressLabel->setText(progressStr);
   mpProgressBar->setRange(0, 0);
   mpProgressBar->setTextVisible(false);
+  updateMessageTab(progressStr);
   mpCancelButton->setText(tr("Cancel Compilation"));
   mpCancelButton->setEnabled(true);
 }
@@ -1140,13 +1183,17 @@ void SimulationOutputWidget::compilationProcessFinished(int exitCode, QProcess::
 void SimulationOutputWidget::simulationProcessStarted()
 {
   mIsSimulationProcessRunning = true;
+  QString progressStr;
   if (mSimulationOptions.isInteractiveSimulation()) {
-    mpProgressLabel->setText(tr("Running interactive simulation of %1.").arg(mSimulationOptions.getClassName()));
+    progressStr = tr("Running interactive simulation of %1.").arg(mSimulationOptions.getClassName());
   } else {
-    mpProgressLabel->setText(tr("Running simulation of %1. Please wait for a while.").arg(mSimulationOptions.getClassName()));
+    progressStr = tr("Running simulation of %1. Please wait for a while.").arg(mSimulationOptions.getClassName());
   }
+  mpProgressLabel->setText(progressStr);
   mpProgressBar->setRange(0, 100);
+  mpProgressBar->setValue(0);
   mpProgressBar->setTextVisible(true);
+  updateMessageTab(progressStr);
   mpCancelButton->setText(Helper::cancelSimulation);
   mpCancelButton->setEnabled(true);
   mpOpenOutputFileButton->setEnabled(true);
@@ -1235,12 +1282,8 @@ void SimulationOutputWidget::openTransformationBrowser(QUrl url)
     /* open the model_info.json file */
     if (QFileInfo(fileName).exists()) {
       TransformationsWidget *pTransformationsWidget = MainWindow::instance()->showTransformationsWidget(fileName, mSimulationOptions.getProfiling().compare(QStringLiteral("none")) != 0);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
       QUrlQuery query(url);
       int equationIndex = query.queryItemValue("index").toInt();
-#else /* Qt4 */
-      int equationIndex = url.queryItemValue("index").toInt();
-#endif
       QTreeWidgetItem *pTreeWidgetItem = pTransformationsWidget->findEquationTreeItem(equationIndex);
       if (pTreeWidgetItem) {
         pTransformationsWidget->getEquationsTreeWidget()->clearSelection();
