@@ -53,17 +53,9 @@
 
 //QT Headers
 #include <QtGlobal>
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
 #include <QtWidgets>
 #include <QDomNode>
 #define fromAscii fromLatin1
-#else
-#include <QtCore/QBuffer>
-#include <QtCore/QFile>
-#include <QtGui/QApplication>
-#include <QtXml/QDomNode>
-#include <QSettings>
-#endif
 
 //IAEX Headers
 #include "xmlparser.h"
