@@ -1,20 +1,21 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-CurrentYear, Linkoping University,
- * Department of Computer and Information Science,
- * SE-58183 Linkoping, Sweden.
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
+ * c/o Linköpings universitet, Department of Computer and Information Science,
+ * SE-58183 Linköping, Sweden.
  *
  * All rights reserved.
  *
- * THIS PROGRAM IS PROVIDED UNDER THE TERMS OF GPL VERSION 3
- * AND THIS OSMC PUBLIC LICENSE (OSMC-PL).
- * ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS PROGRAM CONSTITUTES RECIPIENT'S
- * ACCEPTANCE OF THE OSMC PUBLIC LICENSE.
+ * THIS PROGRAM IS PROVIDED UNDER THE TERMS OF GPL VERSION 3 LICENSE OR
+ * THIS OSMC PUBLIC LICENSE (OSMC-PL) VERSION 1.2.
+ * ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS PROGRAM CONSTITUTES
+ * RECIPIENT'S ACCEPTANCE OF THE OSMC PUBLIC LICENSE OR THE GPL VERSION 3,
+ * ACCORDING TO RECIPIENTS CHOICE.
  *
  * The OpenModelica software and the Open Source Modelica
  * Consortium (OSMC) Public License (OSMC-PL) are obtained
- * from Linkoping University, either from the above address,
+ * from OSMC, either from the above address,
  * from the URLs: http://www.ida.liu.se/projects/OpenModelica or
  * http://www.openmodelica.org, and in the OpenModelica distribution.
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
@@ -22,19 +23,18 @@
  * This program is distributed WITHOUT ANY WARRANTY; without
  * even the implied warranty of  MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
- * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS
- * OF OSMC-PL.
+ * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
  * See the full OSMC Public License conditions for more details.
  *
- * Main Authors 2010: Syed Adeel Asghar, Sonia Tariq
- *
  */
 
-#include <QtSvg/QSvgGenerator>
+#include <iostream>
+#include <memory>
+
 #include "PlotWindow.h"
+#include "LogScaleEngine.h"
 #include "LinearScaleEngine.h"
-#include "iostream"
 #include "qwt_plot_layout.h"
 #if QWT_VERSION >= 0x060000
 #include "qwt_plot_renderer.h"
@@ -44,6 +44,19 @@
 #include "qwt_text_label.h"
 #include "qwt_abstract_legend.h"
 
+#include <QtSvg/QSvgGenerator>
+#include <QToolBar>
+#include <QFileInfo>
+#include <QLayout>
+#include <QCloseEvent>
+#include <QDir>
+#include <QMessageBox>
+#include <QFileDialog>
+#include <QPrinter>
+#include <QPrintDialog>
+#include <QLineEdit>
+#include <QColorDialog>
+
 using namespace OMPlot;
 
 PlotWindow::PlotWindow(QStringList arguments, QWidget *parent, bool isInteractiveSimulation)
@@ -382,7 +395,7 @@ void PlotWindow::setupToolbar()
 void PlotWindow::plot(PlotCurve *pPlotCurve)
 {
   QString currentLine;
-  if (mVariablesList.isEmpty() and getPlotType() == PlotWindow::PLOT)
+  if (mVariablesList.isEmpty() && getPlotType() == PlotWindow::PLOT)
     throw NoVariableException(QString("No variables specified!").toStdString().c_str());
 
   bool editCase = pPlotCurve ? true : false;
@@ -413,7 +426,7 @@ void PlotWindow::plot(PlotCurve *pPlotCurve)
       if (currentLine.contains("DataSet:"))
       {
         currentVariable = currentLine.remove("DataSet: ");
-        if (mVariablesList.contains(currentVariable) or getPlotType() == PlotWindow::PLOTALL)
+        if (mVariablesList.contains(currentVariable) || getPlotType() == PlotWindow::PLOTALL)
         {
           variablesPlotted.append(currentVariable);
           if (!editCase) {
@@ -475,7 +488,7 @@ void PlotWindow::plot(PlotCurve *pPlotCurve)
     // read in all values
     for (int i = 0; i < csvReader->numvars; i++)
     {
-      if (mVariablesList.contains(csvReader->variables[i]) or getPlotType() == PlotWindow::PLOTALL)
+      if (mVariablesList.contains(csvReader->variables[i]) || getPlotType() == PlotWindow::PLOTALL)
       {
         variablesPlotted.append(csvReader->variables[i]);
         double *vals = read_csv_dataset(csvReader, csvReader->variables[i]);
@@ -536,8 +549,8 @@ void PlotWindow::plot(PlotCurve *pPlotCurve)
       throw NoVariableException(QString("Corrupt file. nvar %1").arg(reader.nvar).toStdString().c_str());
     }
     // read in all values
-    for (int i = 0; i < reader.nall; i++) {
-      if (mVariablesList.contains(reader.allInfo[i].name) or getPlotType() == PlotWindow::PLOTALL) {
+    for (uint32_t i = 0; i < reader.nall; i++) {
+      if (mVariablesList.contains(reader.allInfo[i].name) || getPlotType() == PlotWindow::PLOTALL) {
         variablesPlotted.append(reader.allInfo[i].name);
         // create the plot curve for variable
         if (!editCase) {
@@ -562,7 +575,7 @@ void PlotWindow::plot(PlotCurve *pPlotCurve)
             throw NoVariableException(QString("Corrupt file. nvar %1").arg(reader.nvar).toStdString().c_str());
           }
           // set plot curve data and attach it to plot
-          for (int i = 0 ; i < reader.nrows ; i++) {
+          for (uint32_t i = 0 ; i < reader.nrows ; i++) {
             pPlotCurve->addXAxisValue(timeVals[i]);
             pPlotCurve->addYAxisValue(vals[i]);
           }
@@ -782,7 +795,7 @@ void PlotWindow::plotParametric(PlotCurve *pPlotCurve)
           omc_free_matlab4_reader(&reader);
           throw NoVariableException(QString("Corrupt file. nvar %1").arg(reader.nvar).toStdString().c_str());
         }
-        for (int i = 0 ; i < reader.nrows ; i++)
+        for (uint32_t i = 0 ; i < reader.nrows ; i++)
           pPlotCurve->addXAxisValue(xVals[i]);
       }
       // if variable is a parameter then
@@ -809,7 +822,7 @@ void PlotWindow::plotParametric(PlotCurve *pPlotCurve)
           omc_free_matlab4_reader(&reader);
           throw NoVariableException(QString("Corrupt file. nvar %1").arg(reader.nvar).toStdString().c_str());
         }
-        for (int i = 0 ; i < reader.nrows ; i++)
+        for (uint32_t i = 0 ; i < reader.nrows ; i++)
           pPlotCurve->addYAxisValue(yVals[i]);
       }
       // if variable is a parameter then
@@ -874,7 +887,7 @@ int readPLTDataset(QTextStream *mpTextStream, QString variable, int N, double* v
 
 void readPLTArray(QTextStream *mpTextStream, QString variable, double alpha, int intervalSize, int it, QList<double> &arrLstOut){
   int index = 1;
-  double vals[intervalSize];
+  auto vals = std::make_unique<double[]>(intervalSize);
   do  //loop over all indexes
   {
     //read the values
@@ -885,7 +898,7 @@ void readPLTArray(QTextStream *mpTextStream, QString variable, double alpha, int
     } else {
       variableWithInd.append("["+QString::number(index)+"]");
     }
-    if (readPLTDataset(mpTextStream, variableWithInd, intervalSize, vals)){
+    if (readPLTDataset(mpTextStream, variableWithInd, intervalSize, vals.get())){
       if (index == 1)
         throw NoVariableException(QObject::tr("Array variable doesnt exist: %1").arg(variable).toStdString().c_str());
       else
@@ -900,7 +913,8 @@ void readPLTArray(QTextStream *mpTextStream, QString variable, double alpha, int
   return;
 }
 
-double getTimeUnitFactor(QString timeUnit){
+double getTimeUnitFactor(QString timeUnit)
+{
   if (timeUnit == "ms") return 1000.0;
   else if (timeUnit == "s") return 1.0;
   else if (timeUnit == "min") return 1.0/6.0;
@@ -909,8 +923,9 @@ double getTimeUnitFactor(QString timeUnit){
   else throw PlotException(QObject::tr("Unknown unit in plotArray(Parametric)."));
 }
 
-void PlotWindow::updateTimeText(QString unit)
+void PlotWindow::updateTimeText()
 {
+  QString unit = getTimeUnit();
   double timeUnitFactor = getTimeUnitFactor(unit);
   mpPlot->setFooter(QString("t = %1 " + unit).arg(getTime()*timeUnitFactor,0,'g',3));
   mpPlot->replot();
@@ -921,8 +936,7 @@ void PlotWindow::plotArray(double time, PlotCurve *pPlotCurve)
   double *res;
   QString currentLine;
   setTime(time);
-  double timeUnitFactor = getTimeUnitFactor(getTimeUnit());
-  if (mVariablesList.isEmpty() and getPlotType() == PlotWindow::PLOTARRAY)
+  if (mVariablesList.isEmpty() && getPlotType() == PlotWindow::PLOTARRAY)
     throw NoVariableException(QString("No variables specified!").toStdString().c_str());
   bool editCase = pPlotCurve ? true : false;
   //PLOT PLT
@@ -950,11 +964,11 @@ void PlotWindow::plotArray(double time, PlotCurve *pPlotCurve)
     }
     //    double vals[intervalSize];
     //Read in timevector
-    double timeVals[intervalSize];
-    readPLTDataset(mpTextStream, "time", intervalSize, timeVals);
+    auto timeVals = std::make_unique<double[]>(intervalSize);
+    readPLTDataset(mpTextStream, "time", intervalSize, timeVals.get());
     //Find indexes and alpha to interpolate data in particular time
     double alpha;
-    int it = setupInterp(timeVals, time, intervalSize, alpha);
+    int it = setupInterp(timeVals.get(), time, intervalSize, alpha);
     if (it < 0) {
       mFile.close();
       throw PlotException("Time out of bounds.");
@@ -982,8 +996,7 @@ void PlotWindow::plotArray(double time, PlotCurve *pPlotCurve)
       }
       pPlotCurve->setData(pPlotCurve->getXAxisVector(), pPlotCurve->getYAxisVector(), pPlotCurve->getSize());
       pPlotCurve->attach(mpPlot);
-      mpPlot->setFooter(QString("t = %1 " + getTimeUnit()).arg(time*timeUnitFactor,0,'g',3));
-      mpPlot->replot();
+      updateTimeText();
     }
     mFile.close();
   }
@@ -1042,9 +1055,7 @@ void PlotWindow::plotArray(double time, PlotCurve *pPlotCurve)
       }
       pPlotCurve->setData(pPlotCurve->getXAxisVector(), pPlotCurve->getYAxisVector(), pPlotCurve->getSize());
       pPlotCurve->attach(mpPlot);
-      mpPlot->setFooter(QString("t = %1 " + getTimeUnit()).arg(time*timeUnitFactor,0,'g',3));
-      mpPlot->replot();
-
+      updateTimeText();
     }
     omc_free_csv_reader(csvReader);
   }
@@ -1103,8 +1114,7 @@ void PlotWindow::plotArray(double time, PlotCurve *pPlotCurve)
         }
         pPlotCurve->setData(pPlotCurve->getXAxisVector(), pPlotCurve->getYAxisVector(), pPlotCurve->getSize());
         pPlotCurve->attach(mpPlot);
-        mpPlot->setFooter(QString("t = %1 " + getTimeUnit()).arg(time*timeUnitFactor,0,'g',3));
-        mpPlot->replot();
+        updateTimeText();
         delete[] res;
       }
       // if plottype is PLOT then check which requested variables are not found in the file
@@ -1120,7 +1130,6 @@ void PlotWindow::plotArrayParametric(double time, PlotCurve *pPlotCurve)
   QString xVariable, yVariable, xTitle, yTitle;
   int pair = 0;
   setTime(time);
-  double timeUnitFactor = getTimeUnitFactor(getTimeUnit());
   if (mVariablesList.isEmpty())
     throw NoVariableException(QString("No variables specified!").toStdString().c_str());
   else if (mVariablesList.size()%2 != 0)
@@ -1168,11 +1177,11 @@ void PlotWindow::plotArrayParametric(double time, PlotCurve *pPlotCurve)
         throw PlotException(tr("Interval size not specified.").toStdString().c_str());
       }
       //Read in timevector
-      double timeVals[intervalSize];
-      readPLTDataset(mpTextStream, "time", intervalSize, timeVals);
+      auto timeVals = std::make_unique<double[]>(intervalSize);
+      readPLTDataset(mpTextStream, "time", intervalSize, timeVals.get());
       //Find indexes and alpha to interpolate data in particular time
       double alpha;
-      int it = setupInterp(timeVals, time, intervalSize, alpha);
+      int it = setupInterp(timeVals.get(), time, intervalSize, alpha);
       if (it < 0) {
         mFile.close();
         throw PlotException("Time out of bounds.");
@@ -1200,8 +1209,7 @@ void PlotWindow::plotArrayParametric(double time, PlotCurve *pPlotCurve)
       }
       pPlotCurve->setData(pPlotCurve->getXAxisVector(), pPlotCurve->getYAxisVector(), pPlotCurve->getSize());
       pPlotCurve->attach(mpPlot);
-      mpPlot->setFooter(QString("t = %1 " + getTimeUnit()).arg(time*timeUnitFactor,0,'g',3));
-      mpPlot->replot();
+      updateTimeText();
       mFile.close();
     }
     //    //PLOT CSV
@@ -1238,7 +1246,7 @@ void PlotWindow::plotArrayParametric(double time, PlotCurve *pPlotCurve)
       for (int j = 0; j<2; j++){
         res.clear();
         double *arrElement;
-        for (int i = 1; i++; true){
+        for (int i = 1; ;i++){
           QString varNameQS = varPair[j];
           if (QRegExp("der\\(\\D(\\w)*\\)").exactMatch(varNameQS)){
             varNameQS.chop(1);
@@ -1268,8 +1276,7 @@ void PlotWindow::plotArrayParametric(double time, PlotCurve *pPlotCurve)
       }
       pPlotCurve->setData(pPlotCurve->getXAxisVector(), pPlotCurve->getYAxisVector(), pPlotCurve->getSize());
       pPlotCurve->attach(mpPlot);
-      mpPlot->setFooter(QString("t = %1 " + getTimeUnit()).arg(time*timeUnitFactor,0,'g',3));
-      mpPlot->replot();
+      updateTimeText();
       omc_free_csv_reader(csvReader);
     }
     //PLOT MAT
@@ -1341,8 +1348,7 @@ void PlotWindow::plotArrayParametric(double time, PlotCurve *pPlotCurve)
       }
       pPlotCurve->setData(pPlotCurve->getXAxisVector(), pPlotCurve->getYAxisVector(), pPlotCurve->getSize());
       pPlotCurve->attach(mpPlot);
-      mpPlot->setFooter(QString("t = %1 " + getTimeUnit()).arg(time*timeUnitFactor,0,'g',3));
-      mpPlot->replot();
+      updateTimeText();
       omc_free_matlab4_reader(&reader);
     }
   }
@@ -1799,44 +1805,38 @@ void PlotWindow::fitInView()
 
 void PlotWindow::setLogX(bool on)
 {
-  if(on)
-  {
+  if (on) {
 #if QWT_VERSION >= 0x060100
-    mpPlot->setAxisScaleEngine(QwtPlot::xBottom, new QwtLogScaleEngine);
+    mpPlot->setAxisScaleEngine(QwtPlot::xBottom, new LogScaleEngine);
 #else
     mpPlot->setAxisScaleEngine(QwtPlot::xBottom, new QwtLog10ScaleEngine);
 #endif
-  }
-  else
-  {
+  } else {
     mpPlot->setAxisScaleEngine(QwtPlot::xBottom, new LinearScaleEngine);
   }
   mpPlot->setAxisAutoScale(QwtPlot::xBottom);
-  mpLogXCheckBox->blockSignals(true);
+  bool state = mpLogXCheckBox->blockSignals(true);
   mpLogXCheckBox->setChecked(on);
-  mpLogXCheckBox->blockSignals(false);
-  mpPlot->replot();
+  mpLogXCheckBox->blockSignals(state);
+  updatePlot();
 }
 
 void PlotWindow::setLogY(bool on)
 {
-  if(on)
-  {
+  if (on) {
 #if QWT_VERSION >= 0x060100
-    mpPlot->setAxisScaleEngine(QwtPlot::yLeft, new QwtLogScaleEngine);
+    mpPlot->setAxisScaleEngine(QwtPlot::yLeft, new LogScaleEngine);
 #else
     mpPlot->setAxisScaleEngine(QwtPlot::yLeft, new QwtLog10ScaleEngine);
 #endif
-  }
-  else
-  {
+  } else {
     mpPlot->setAxisScaleEngine(QwtPlot::yLeft, new LinearScaleEngine);
   }
   mpPlot->setAxisAutoScale(QwtPlot::yLeft);
-  mpLogYCheckBox->blockSignals(true);
+  bool state = mpLogYCheckBox->blockSignals(true);
   mpLogYCheckBox->setChecked(on);
-  mpLogYCheckBox->blockSignals(false);
-  mpPlot->replot();
+  mpLogYCheckBox->blockSignals(state);
+  updatePlot();
 }
 
 void PlotWindow::setAutoScale(bool on)
