@@ -30,14 +30,15 @@
  */
 
 #include "Scope.h"
-
+#include "XercesValidator.h"
 #include "Flags.h"
 #include "System.h"
 #include "Component.h"
 #include "Snapshot.h"
-#include <miniunz.h>
+#include "miniunz.h"
 #include <time.h>
 #include "ssd/Tags.h"
+#include <iostream>
 
 oms::Scope::Scope()
   : tempDir(".")
@@ -141,12 +142,10 @@ oms_status_enu_t oms::Scope::exportModel(const oms::ComRef& cref, const std::str
   return model->exportToFile(filename);
 }
 
-oms_status_enu_t oms::Scope::miniunz(const std::string& filename, const std::string& extractdir, bool systemStructure)
+oms_status_enu_t oms::Scope::miniunz(const std::string& filename, const std::string& extractdir)
 {
-  // This function is used to extract (1) complete SSP/FMU files or to extract
-  // (2) the SystemStructure.ssd from an SSP file:
-  //     (1) miniunz -xo <filename> -d <extractdir>
-  //     (2) miniunz -xo <filename> SystemStructure.ssd -d <extractdir>
+  // This function is used to extract complete SSP/FMU files:
+  //        miniunz -xo <filename> -d <extractdir>
 
   // Usage: miniunz [-e] [-x] [-v] [-l] [-o] [-p password] file.zip [file_to_extr.] [-d extractdir]
   //        -e  Extract without pathname (junk paths)
@@ -159,14 +158,12 @@ oms_status_enu_t oms::Scope::miniunz(const std::string& filename, const std::str
 
   std::string cd = Scope::GetInstance().getWorkingDirectory();
 
-  int argc = systemStructure ? 6 : 5;
+  int argc = 5;
   char **argv = new char*[argc];
   int i=0;
   argv[i++] = (char*)"miniunz";
   argv[i++] = (char*)"-xo";
   argv[i++] = (char*)filename.c_str();
-  if (systemStructure)
-    argv[i++] = (char*)"SystemStructure.ssd";
   argv[i++] = (char*)"-d";
   argv[i++] = (char*)extractdir.c_str();
   int status = ::miniunz(argc, argv);
@@ -195,13 +192,20 @@ oms_status_enu_t oms::Scope::importModel(const std::string& filename, char** _cr
   if (extension != ".ssp")
     return logError("filename extension must be \".ssp\"; no other formats are supported");
 
-  // extract SystemStructure.ssd to temp
-  filesystem::path temp_root(getTempDirectory());
-  if (oms_status_ok != oms::Scope::miniunz(filename, temp_root.string(), true))
-    return logError("failed to extract \"SystemStructure.ssd\" from \"" + filename + "\"");
+  // extract SystemStructure.ssd to memory
+  const char* systemStructure = ::miniunz_onefile_to_memory(filename.c_str(), "SystemStructure.ssd");
+  if (!systemStructure)
+    return logError("failed to extract \"SystemStructure.ssd\" from \"" + std::string(filename) + "\"");
+
+  XercesValidator xercesValidator;
+  xercesValidator.validateSSP(systemStructure, filename);
 
   Snapshot snapshot;
-  snapshot.importResourceFile("SystemStructure.ssd", temp_root);
+  oms_status_enu_t status = snapshot.importResourceMemory("SystemStructure.ssd", systemStructure);
+  ::miniunz_free(systemStructure);
+  if (oms_status_ok != status)
+    return logError("Failed to import");
+
   const pugi::xml_node node = snapshot.getResourceNode("SystemStructure.ssd");
   if (!node)
     return logError("failed to load \"SystemStructure.ssd\"");
@@ -217,7 +221,7 @@ oms_status_enu_t oms::Scope::importModel(const std::string& filename, char** _cr
     logWarning("Unknown SSD version: " + ssdVersion);
 
   // extract the ssp file
-  oms::Scope::miniunz(filename, model->getTempDirectory(), false);
+  oms::Scope::miniunz(filename, model->getTempDirectory());
 
   std::string cd = Scope::GetInstance().getWorkingDirectory();
   Scope::GetInstance().setWorkingDirectory(model->getTempDirectory());
@@ -228,12 +232,31 @@ oms_status_enu_t oms::Scope::importModel(const std::string& filename, char** _cr
   // add the remaining resources (e.g) .ssv, .ssm and signalFilter.xml to oms:snapshot
   for (const auto &entry : OMS_RECURSIVE_DIRECTORY_ITERATOR(model->getTempDirectory()))
     if (entry.path().has_extension())
-      if (".ssv" == entry.path().extension() || ".ssm" == entry.path().extension() || ".xml" == entry.path().extension())
-        snapshot.importResourceFile(naive_uncomplete(entry.path(), model->getTempDirectory()), model->getTempDirectory());
+    {
+      // TODO collect all the files and sort the ssd variant, because the iteration order is different for windows and linux
+      // check for ssd variants
+      if (".ssd" == entry.path().extension())
+      {
+        if (entry.path().filename() != "SystemStructure.ssd")
+        {
+          //std::cout << "\n Debug print : " << entry.path().filename().generic_string();
+          Snapshot variants;
+          variants.importResourceFile(naive_uncomplete(entry.path(), model->getTempDirectory()), model->getTempDirectory());
+          char* variantSnapshot = NULL;
+          variants.writeDocument(&variantSnapshot);
+          model->ssdVariants[variants.getRootCref()] = variantSnapshot;
+        }
+      }
 
+      if (".ssv" == entry.path().extension() || ".ssm" == entry.path().extension() || ".xml" == entry.path().extension())
+      {
+        model->importedResources.push_back(entry.path().filename().generic_string());
+        snapshot.importResourceFile(naive_uncomplete(entry.path(), model->getTempDirectory()), model->getTempDirectory());
+      }
+    }
   // snapshot.debugPrintAll();
 
-  oms_status_enu_t status = model->importFromSnapshot(snapshot);
+  status = model->importFromSnapshot(snapshot);
   model->copyResources(old_copyResources);
 
   Scope::GetInstance().setWorkingDirectory(cd);
@@ -270,15 +293,7 @@ oms_status_enu_t oms::Scope::setTempDirectory(const std::string& newTempDir)
     }
 
     filesystem::path path(newTempDirWorkaround.c_str());
-    try
-    {
-      path = oms_canonical(path);
-    }
-    catch (std::exception e)
-    {
-      // do nothing, canonical fails if the directory contains a junction or a symlink!
-      // https://svn.boost.org/trac10/ticket/11138
-    }
+    path = oms_canonical(path);
 
     this->tempDir = path.string();
 
@@ -297,19 +312,18 @@ oms_status_enu_t oms::Scope::setWorkingDirectory(const std::string& newWorkingDi
 {
   try
   {
-    if (!filesystem::is_directory(newWorkingDir))
-      return logError("Set working directory to \"" + newWorkingDir + "\" failed");
+    std::string newWorkingDirWorkaround = newWorkingDir;
+#if defined(__MINGW32__)
+    // create_directory() behaves different with clang 11 than gcc for path with trailing "/"
+    if('/' == newWorkingDirWorkaround.back()) {
+      newWorkingDirWorkaround.pop_back();
+    }
+#endif
+    if (!filesystem::is_directory(newWorkingDirWorkaround))
+      return logError("Set working directory to \"" + newWorkingDirWorkaround + "\" failed");
 
-    filesystem::path path(newWorkingDir.c_str());
-    try
-    {
-      path = oms_canonical(path);
-    }
-    catch (const std::exception&)
-    {
-      // do nothing, canonical fails if the directory contains a junction or a symlink!
-      // https://svn.boost.org/trac10/ticket/11138
-    }
+    filesystem::path path(newWorkingDirWorkaround.c_str());
+    path = oms_canonical(path);
 
     filesystem::current_path(path);
 
